require 'active_support/core_ext/hash/deep_merge'

describe 'search' do
  
  def link id, page, per_page
    "https://us-east-1.s3searchapp.test/v1/searches/abcde.json?page=#{page}&per_page=#{per_page}"
  end

  let(:found_documents) { (1..70).map{|i| {id: SecureRandom.hex, name: "doc#{i}", _score: 1.0 / i}}}

  let(:simple_request_body) {
    { 
      search: {
        query: {
          bool: {
            must: [
              {
                query_string:{
                  default_field: '_all',
                  query: "searchterm",
                  default_operator:"AND"
                }
              }
            ]
          }
        }
      },
      page: 1,
      per_page: 25
    }
  }
  it 'finds by simple query' do      
    stub_api_http :post, "/v1/searches.json", simple_request_body do
      {
        body: Hashie::Mash.new(
          total: 70,
          from: 1,
          to: 25,
          results: found_documents[0..24],
          links: {
            self: link('abcde', 1, 25),
            next: link('abcde', 2, 25)
          }
        ),
        status: 200
      }
    end

    results = S3Search.simple_search 'searchterm'
    expect(results.first).to eq(
      {
        "_score" => 1.0,
        "id" => results[0].id,
        "name" => "doc1"
      }
    )
    expect(results.last).to eq(
      {
        "_score" => 0.04,
        "id" => results[24].id,
        "name" => "doc25"
      }
    )

    stub_api_http :get, "/v1/searches/abcde.json?page=2&per_page=25" do
      {
        body: Hashie::Mash.new(
          total: 70,
          from: 26,
          to: 50,
          results: found_documents[25..49],
          links: {
            self: link('abcde', 2, 25),
            next: link('abcde', 3, 25),
            prev: link('abcde', 1, 25)
          }
        ),
        status: 200
      }
    end

    results.next
    expect(results.first).to eq(
      {
        "_score" => results[0]._score,
        "id" => results[0].id,
        "name" => "doc26"
      }
    )
    expect(results.last).to eq(
      {
        "_score" => results[24]._score,
        "id" => results[24].id,
        "name" => "doc50"
      }
    )

  end

  let(:highlight_options) {
    {
      highlight: {
        fields: [:name],
        tag: '<strong class="highlight">',
        number_of_fragments: 3,
        fragment_size: 150          
      }
    }
  }
  let(:highlight_request_body) {
    simple_request_body.deep_merge(search: highlight_options)
  }

  let(:highlight_documents) { 
    (1..70).map do |i| 
      {
        id: SecureRandom.hex, 
        name: "doc#{i}", 
        description: 'a searchterm here',
        _score: 1.0 / i,
        _highlight: {
          _description: ['a <strong class="highlight">searchterm</strong> here']
        }
      }
    end
  }

  it 'allows highlighting of fields' do
    stub_api_http :post, "/v1/searches.json", highlight_request_body do
      {
        body: Hashie::Mash.new(
          total: 70,
          from: 1,
          to: 25,
          results: highlight_documents[0..24],
          links: {
            self: link('abcde', 1, 25),
            next: link('abcde', 2, 25)
          }
        ),
        status: 200
      }
    end
    results = S3Search.simple_search 'searchterm', highlight_options
    expect(results.first).to eq({
      "_highlight" => {"_description"=>['a <strong class="highlight">searchterm</strong> here']},
      "_score" => 1.0,
      "description" => "a searchterm here",
      "id" => results[0].id,
      "name" => "doc1"
      })
  end
  
end
